@@ -501,7 +501,7 @@ iiop_connection_server(void)
   GIOP_CONNECTION(server_cnx)->fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 
   if(GIOP_CONNECTION_GET_FD(server_cnx) < 0) {
-    ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
+    ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_server: socket_error: %s\n", strerror(errno));
     goto failed;
   }
 
@@ -579,7 +579,7 @@ iiop_connection_server_ipv6(void)
   GIOP_CONNECTION(server_cnx)->fd = socket(PF_INET6, SOCK_STREAM, IPPROTO_TCP);
 
   if(GIOP_CONNECTION_GET_FD(server_cnx) < 0) {
-    ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
+    ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_server_ipv6: socket_error: %s\n", strerror(errno));
     goto failed;
   }
 
@@ -656,7 +656,7 @@ iiop_connection_server_unix(const char *sockpath)
   GIOP_CONNECTION(server_cnx_unix)->fd = socket(AF_UNIX, SOCK_STREAM, 0);
 
   if(GIOP_CONNECTION_GET_FD(server_cnx_unix) < 0) {
-    ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
+    ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_server_unix: socket_error: %s\n", strerror(errno));
     goto failed;
   }
 
@@ -848,6 +848,12 @@ iiop_connection_new(const char *host, gushort port)
 	      GIOP_CONNECTION_GET_FD(retval),
 	      host, (guint)port);
 
+
+  fcntl(GIOP_CONNECTION_GET_FD(retval), F_SETFD, FD_CLOEXEC);
+  fcntl(GIOP_CONNECTION_GET_FD(retval), F_SETFL,
+	fcntl(GIOP_CONNECTION_GET_FD(retval), F_GETFL, 0)
+	| O_NONBLOCK);
+
   GIOP_CONNECTION(retval)->was_initiated = TRUE;
   GIOP_CONNECTION(retval)->is_auth = TRUE;
 
@@ -950,6 +956,11 @@ iiop_connection_unix_new(const char *sockpath)
   GIOP_CONNECTION(retval)->was_initiated = TRUE;
   GIOP_CONNECTION(retval)->is_auth = TRUE;
 
+  fcntl(GIOP_CONNECTION_GET_FD(retval), F_SETFD, FD_CLOEXEC);
+  fcntl(GIOP_CONNECTION_GET_FD(retval), F_SETFL,
+	fcntl(GIOP_CONNECTION_GET_FD(retval), F_GETFL, 0)
+	| O_NONBLOCK);
+
   giop_connection_add_to_list(GIOP_CONNECTION(retval));
 
   ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug,
