@@ -462,7 +462,8 @@ giop_send_locate_request_buffer_use(GIOPConnection *connection,
 void
 giop_send_buffer_unuse(GIOPSendBuffer *send_buffer)
 {
-  g_return_if_fail(send_buffer != NULL);
+  if (send_buffer == NULL)
+    return;
 
   giop_connection_unref(GIOP_MESSAGE_BUFFER(send_buffer)->connection);
 
@@ -715,7 +716,8 @@ giop_recv_buffer_new(void)
 void
 giop_recv_buffer_unuse(GIOPRecvBuffer *buffer)
 {
-  g_return_if_fail(buffer != NULL);
+  if (buffer == NULL)
+    return;
 
   if(buffer->message_body) {
     buffer->message_body = ((guchar *)buffer->message_body)
