@@ -357,7 +357,7 @@ ORBit_POA_remove_child(PortableServer_POA poa,
 	poa->child_POAs = g_slist_remove(poa->child_POAs, child_poa);
 }
 
-void
+gboolean
 ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 			 PortableServer_POA poa)
 {
@@ -380,7 +380,7 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 	case PortableServer_POAManager_HOLDING:
 		poa->held_requests = g_slist_prepend(poa->held_requests,
 						     recv_buffer);
-		return;
+		return FALSE;
 		break;
 	case PortableServer_POAManager_DISCARDING:
 		send_buffer = giop_send_reply_buffer_use(GIOP_MESSAGE_BUFFER(recv_buffer)->connection,
@@ -395,7 +395,7 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 		giop_recv_buffer_unuse(recv_buffer);
 		giop_send_buffer_unuse(send_buffer);
 		CORBA_exception_free(&ev);
-		return;
+		return TRUE;
 		break;
 	case PortableServer_POAManager_INACTIVE:
 		send_buffer = giop_send_reply_buffer_use(GIOP_MESSAGE_BUFFER(recv_buffer)->connection,
@@ -410,7 +410,7 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 		giop_recv_buffer_unuse(recv_buffer);
 		giop_send_buffer_unuse(send_buffer);
 		CORBA_exception_free(&ev);
-		return;
+		return TRUE;
 		break;
 	case PortableServer_POAManager_ACTIVE:
 	default:
@@ -491,7 +491,7 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 						       cookie, &ev);
 	}
 
-	return;
+	return TRUE;
 
  errout:
 	if(ev._major == CORBA_SYSTEM_EXCEPTION) {
@@ -511,6 +511,8 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 		g_assert(ev._major == CORBA_NO_EXCEPTION);
 
 	CORBA_exception_free(&ev);
+
+	return TRUE;
 }
 
 PortableServer_POA
